/*
 * Copyright (C), 2000-2003 by Contributors to the monit codebase. 
 * All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <config.h>

#include <stdio.h>
#include <stdlib.h>

#include "monitor.h"
#include "protocol.h"
#include "ssl.h"


/* Private prototypes */
static void _gcppl(Port_T*);
static void _gcpl(Service_T*);
static void _gcpcl(Checksum_T*);
static void _gcpql(Resource_T*);
static void _gcppil(ProcInfo_T*);
static void _gcpdil(DeviceInfo_T*);
static void _gcptl(Timestamp_T*);
static void _gcpdl(Dependant_T *d);


/**
 *  Release allocated memory.
 *
 *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
 *  @author Martin Pala, <martin.pala@iol.cz>
 *
 *  @version \$Id: gc.c,v 1.20 2003/06/18 01:47:52 hauk Exp $
 *
 *  @file
 */


/* ------------------------------------------------------------------ Public */


void gc() {

  gc_protocols();
  if(servicelist) _gcpl(&servicelist);
  if(Run.mygroup) free(Run.mygroup);
  
}


void gc_service(Service_T *s) {

  int i;

  ASSERT(s&&*s);
  
  if((*s)->portlist) {
    
    _gcppl(&(*s)->portlist);

  }

  if((*s)->checksumlist) {
    
    _gcpcl(&(*s)->checksumlist);

  }

  if((*s)->maillist) {

    gc_mail_list(&(*s)->maillist);

  }

  if((*s)->resourcelist) {
    
    _gcpql(&(*s)->resourcelist);
    
  }

  if((*s)->procinfo) {

    _gcppil(&(*s)->procinfo);

  }
  
  if((*s)->devinfo) {

    _gcpdil(&(*s)->devinfo);

  }
  
  if((*s)->timestamplist) {
    
    _gcptl(&(*s)->timestamplist);

  }

  if((*s)->dependantlist) {
    
    _gcpdl(&(*s)->dependantlist);
    
  }

  free((*s)->name);
  free((*s)->path);
  free((*s)->group);
  
  if((*s)->start) {
    for(i= 0; (*s)->start->arg[i]; i++)
	free((*s)->start->arg[i]);
    free((*s)->start);
  }
  
  if((*s)->stop) {
    for(i= 0; (*s)->stop->arg[i]; i++)
	free((*s)->stop->arg[i]);
    free((*s)->stop);
  }
  
  (*s)->next= NULL;

  pthread_mutex_destroy(&(*s)->mutex);

  free(*s);

  *s= NULL;

}
  

void gc_mail_list(Mail_T *m) {

  ASSERT(m);
  
  if((*m)->next)
      gc_mail_list(&(*m)->next);
  
  free((*m)->to);
  free((*m)->from);
  free((*m)->subject);
  free((*m)->message);
  free((*m)->opt_message);
  free(*m);
  *m= NULL;
  
}

    
/* ----------------------------------------------------------------- Private */


static void _gcpl(Service_T *s) {
  
  ASSERT(s&&*s);

  if((*s)->next) {
    
    _gcpl(&(*s)->next);
    
  }
  
  gc_service(&(*s));
  *s= NULL;
    
}


static void _gcppl(Port_T *s) {
  
  ASSERT(s&&*s);

  if((*s)->next) {
    
    _gcppl(&(*s)->next);
    
  }

  free((*s)->hostname);
  free((*s)->request);
  free((*s)->pathname);
  free((*s)->address);
  free((*s)->certmd5);
 
  delete_ssl_socket((*s)->ssl);
  free(*s);
  *s= NULL;

}


static void _gcpcl(Checksum_T *s) {
  
  ASSERT(s&&*s);

  if((*s)->next) {
    
    _gcpcl(&(*s)->next);
    
  }

  free((*s)->file); 
  free((*s)->md5);
  free(*s);
  *s= NULL;

}


static void _gcpql(Resource_T *q) {

  ASSERT(q);

  if((*q)->next) {

    _gcpql(&(*q)->next);

  }

  free(*q);
  *q= NULL;
  
}


static void _gcppil(ProcInfo_T *pi) {

  ASSERT(pi);

  free(*pi);
  *pi= NULL;

}


static void _gcpdil(DeviceInfo_T *di) {

  ASSERT(di);

  free(*di);
  *di= NULL;

}


static void _gcptl(Timestamp_T *p) {
  
  ASSERT(p);

  if((*p)->next) {
    
    _gcptl(&(*p)->next);
    
  }

  free((*p)->pathname);
  free(*p);
  *p= NULL;

}


static void _gcpdl(Dependant_T *d) {

  ASSERT(d);

  if((*d)->next) {
    
    _gcpdl(&(*d)->next);
    
  }

  free((*d)->dependant);
  free(*d);
  *d= NULL;

}
